@@ -1167,8 +1167,7 @@ class Banana(protocol.Protocol):
 
     def handleError(self, msg):
         log.msg("got banana ERROR from remote side: %s" % msg)
-        e = BananaError("remote error: %s" % msg)
-        self.transport.loseConnection(Failure(e))
+        self.transport.loseConnection()
 
 
     def describeReceive(self):
