@@ -9,6 +9,8 @@ from foolscap.logging import levels, app_versions
 from foolscap.eventual import eventually
 from foolscap import base32
 
+TIME_FORMAT = "%Y-%m-%d--%H-%M-%S"
+
 class IncidentQualifier:
     """I am responsible for deciding what qualifies as an Incident. I look at
     the event stream and watch for a 'triggering event', then signal my
@@ -56,7 +58,6 @@ class IncidentReporter:
 
     TRAILING_DELAY = 5.0 # gather 5 seconds of post-trigger events
     TRAILING_EVENT_LIMIT = 100 # or 100 events, whichever comes first
-    TIME_FORMAT = "%Y-%m-%d--%H:%M:%S"
 
     def __init__(self, basedir, logger, tubid_s):
         self.basedir = basedir
@@ -68,7 +69,7 @@ class IncidentReporter:
         return self.active
 
     def format_time(self, when):
-        return time.strftime(self.TIME_FORMAT, time.gmtime(when)) + "Z"
+        return time.strftime(TIME_FORMAT, time.gmtime(when)) + "Z"
 
     def incident_declared(self, triggering_event):
         self.trigger = triggering_event
