@@ -1,5 +1,5 @@
 
-import os.path
+import sys, os.path
 from StringIO import StringIO
 from twisted.trial import unittest
 from twisted.internet import defer
@@ -128,6 +128,28 @@ class CLI(RequiresCryptoBase, unittest.TestCase):
         d.addCallback(_check_add)
         return d
 
+    def test_add_service(self):
+        basedir = "appserver/CLI/add_service"
+        os.makedirs(basedir)
+        serverdir = os.path.join(basedir, "fl")
+        incomingdir = os.path.join(basedir, "incoming")
+        os.mkdir(incomingdir)
+        d = self.run_cli("create", serverdir)
+        def _check((rc,out,err)):
+            self.failUnlessEqual(rc, 0)
+            self.failUnless(os.path.isdir(serverdir))
+        d.addCallback(_check)
+        def _check_add(ign):
+            furl,servicedir = cli.add_service(serverdir,
+                                              "upload-file", (incomingdir,),
+                                              None)
+            swiss = furl[furl.rfind("/")+1:]
+            servicedir2 = os.path.join(serverdir, "services", swiss)
+            self.failUnlessEqual(os.path.abspath(servicedir),
+                                 os.path.abspath(servicedir2))
+        d.addCallback(_check_add)
+        return d
+
     def test_add_comment(self):
         basedir = "appserver/CLI/add_comment"
         os.makedirs(basedir)
@@ -214,6 +236,15 @@ class CLI(RequiresCryptoBase, unittest.TestCase):
         def _check_add((rc,out,err)):
             self.failUnlessEqual(rc, 0)
         d.addCallback(_check_add)
+
+        def _check_list_services(ign):
+            services = cli.list_services(serverdir)
+            self.failUnlessEqual(len(services), 1)
+            s = services[0]
+            self.failUnlessEqual(s.service_type, "upload-file")
+            self.failUnlessEqual(s.service_args, (incomingdir,) )
+        d.addCallback(_check_list_services)
+
         d.addCallback(lambda ign: self.run_cli("list", serverdir))
         def _check_list((rc,out,err)):
             self.failUnlessEqual(rc, 0)
@@ -432,22 +463,37 @@ class Upload(RequiresCryptoBase, unittest.TestCase, ShouldFailMixin):
         f.write(DATA4)
         f.close()
 
+        sourcefile5 = os.path.join(basedir, "file5.txt")
+        f = open(sourcefile5, "wb")
+        DATA5 = "file number 5\n"
+        f.write(DATA5)
+        f.close()
+
         d.addCallback(lambda _ign:
                       self.run_client("--furl", self.furl, "upload-file",
-                                      sourcefile3, sourcefile4))
+                                      sourcefile3, sourcefile4, sourcefile5))
         def _check_client4((rc,out,err)):
             self.failUnlessEqual(rc, 0)
             self.failUnlessIn("file3.txt: uploaded", out)
             self.failUnlessIn("file4.txt: uploaded", out)
+            self.failUnlessIn("file5.txt: uploaded", out)
             self.failUnlessEqual(err.strip(), "")
+
             fn = os.path.join(incomingdir, "file3.txt")
             self.failUnless(os.path.exists(fn))
             contents = open(fn,"rb").read()
             self.failUnlessEqual(contents, DATA3)
+
             fn = os.path.join(incomingdir, "file4.txt")
             self.failUnless(os.path.exists(fn))
             contents = open(fn,"rb").read()
             self.failUnlessEqual(contents, DATA4)
+
+            fn = os.path.join(incomingdir, "file5.txt")
+            self.failUnless(os.path.exists(fn))
+            contents = open(fn,"rb").read()
+            self.failUnlessEqual(contents, DATA5)
+
         d.addCallback(_check_client4)
 
         return d
@@ -558,12 +604,14 @@ class RunCommand(unittest.TestCase, RequiresCryptoBase, StallMixin):
             f.close()
         d.addCallback(_populate_foo)
 
+        helper = os.path.join(os.path.dirname(__file__), "apphelper.py")
         d.addCallback(lambda ign:
                       self.add(serverdir,
                                "run-command",
                                "--no-log-stdin", "--log-stdout",
                                "--no-log-stderr",
-                               incomingdir, "cat", "foo.txt"))
+                               incomingdir,
+                               sys.executable, helper, "cat", "foo.txt"))
         d.addCallback(self.stash_furl, 0)
         stdout = StringIO()
         def _start_server(ign):
@@ -598,7 +646,7 @@ class RunCommand(unittest.TestCase, RequiresCryptoBase, StallMixin):
                                "run-command", "--accept-stdin",
                                "--log-stdin", "--no-log-stdout", "--log-stderr",
                                incomingdir,
-                               "dd", "of=bar.txt"))
+                               sys.executable, helper, "dd", "of=bar.txt"))
         d.addCallback(self.stash_furl, 1)
 
         barfile = os.path.join(incomingdir, "bar.txt")
@@ -610,7 +658,7 @@ class RunCommand(unittest.TestCase, RequiresCryptoBase, StallMixin):
             self.failUnlessEqual(rc, 0)
             bardata = open(barfile,"rb").read()
             self.failUnlessEqual(bardata, DATA2)
-            # these checks depend upon /bin/dd behaving consistently
+            # we use a script instead of the real dd; we know how it behaves
             self.failUnlessEqual(out, "")
             self.failUnlessIn("records in", err.strip())
         d.addCallback(_check_client3)
@@ -622,7 +670,7 @@ class RunCommand(unittest.TestCase, RequiresCryptoBase, StallMixin):
                                "run-command",
                                "--no-stdin", "--send-stdout", "--no-stderr",
                                incomingdir,
-                               "cat", "foo.txt"))
+                               sys.executable, helper, "cat", "foo.txt"))
         d.addCallback(self.stash_furl, 2)
 
         d.addCallback(lambda ign:
@@ -630,11 +678,11 @@ class RunCommand(unittest.TestCase, RequiresCryptoBase, StallMixin):
                                "run-command",
                                "--no-stdin", "--no-stdout", "--send-stderr",
                                incomingdir,
-                               "cat", "foo.txt"))
+                               sys.executable, helper, "cat", "foo.txt"))
         d.addCallback(self.stash_furl, 3)
 
         d.addCallback(_populate_foo)
-            
+
         d.addCallback(lambda _ign:
                       self.run_client("--furl", self.furls[2], "run-command"))
         def _check_client4((rc,out,err)):
