@@ -11,6 +11,7 @@ import foolscap
 from foolscap.logging import gatherer, log, tail, incident, cli, web, \
      publish, dumper
 from foolscap.logging.interfaces import RILogObserver
+from foolscap.util import format_time
 from foolscap.eventual import fireEventually, flushEventualQueue
 from foolscap.tokens import NoLocationError
 from foolscap.test.common import PollMixin, StallMixin, GoodEnoughTub
@@ -518,7 +519,7 @@ class Publish(PollMixin, unittest.TestCase):
                 tw_log.msg("message 2 here")
 
                 # switch to generic (no tubid) bridge
-                tw_log.removeObserver(tlb.observer)
+                log.unbridgeLogsFromTwisted(tw_log, tlb)
                 log.bridgeLogsFromTwisted(None, tw_log)
 
                 tw_log.msg("message 3 here")
@@ -1302,11 +1303,63 @@ class Gatherer(unittest.TestCase, LogfileReaderMixin, StallMixin, PollMixin):
         # gatherer.d will be fired when subscribe_to_all() has finished
         d = gatherer.d
         d.addCallback(self._emit_messages_and_flush, t)
+        # We use do_rotate() to force logfile rotation before checking
+        # contents of the file, so we know it's been written out to disk
         d.addCallback(lambda res: gatherer.do_rotate())
         d.addCallback(self._check_gatherer, starting_timestamp, expected_tubid)
         return d
     test_log_gatherer.timeout = 20
 
+    def test_log_gatherer_multiple(self):
+        # setLocation, then set log-gatherer-furl.
+        basedir = "logging/Gatherer/log_gatherer_multiple"
+        os.makedirs(basedir)
+
+        # create a gatherer, which will create its own Tub
+        gatherer1_basedir = os.path.join(basedir, "gatherer1")
+        os.makedirs(gatherer1_basedir)
+        gatherer1 = MyGatherer(None, False, gatherer1_basedir)
+        gatherer1.tub_class = GoodEnoughTub
+        gatherer1.d = defer.Deferred()
+        gatherer1.setServiceParent(self.parent)
+        # that will start the gatherer
+        gatherer1_furl = gatherer1.my_furl
+        starting_timestamp1 = gatherer1._starting_timestamp
+
+        # create a second one
+        gatherer2_basedir = os.path.join(basedir, "gatherer2")
+        os.makedirs(gatherer2_basedir)
+        gatherer2 = MyGatherer(None, False, gatherer2_basedir)
+        gatherer2.tub_class = GoodEnoughTub
+        gatherer2.d = defer.Deferred()
+        gatherer2.setServiceParent(self.parent)
+        # that will start the gatherer
+        gatherer2_furl = gatherer2.my_furl
+        starting_timestamp2 = gatherer2._starting_timestamp
+
+        t = GoodEnoughTub()
+        expected_tubid = t.tubID
+        if t.tubID is None:
+            expected_tubid = "<unauth>"
+        t.setServiceParent(self.parent)
+        l = t.listenOn("tcp:0:interface=127.0.0.1")
+        t.setLocation("127.0.0.1:%d" % l.getPortnum())
+        t.setOption("log-gatherer-furl", (gatherer1_furl, gatherer2_furl))
+
+        # about now, the node will be contacting the Gatherers and
+        # offering its logport.
+
+        # gatherer.d and gatherer2.d will be fired when subscribe_to_all()
+        # has finished
+        dl = defer.DeferredList([gatherer1.d, gatherer2.d])
+        dl.addCallback(self._emit_messages_and_flush, t)
+        dl.addCallback(lambda res: gatherer1.do_rotate())
+        dl.addCallback(self._check_gatherer, starting_timestamp1, expected_tubid)
+        dl.addCallback(lambda res: gatherer2.do_rotate())
+        dl.addCallback(self._check_gatherer, starting_timestamp2, expected_tubid)
+        return dl
+    test_log_gatherer_multiple.timeout = 40
+
     def test_log_gatherer2(self):
         # set log-gatherer-furl, then setLocation. Also, use a timed rotator.
         basedir = "logging/Gatherer/log_gatherer2"
@@ -1507,7 +1560,8 @@ class Tail(unittest.TestCase):
     def test_logprinter(self):
         target_tubid_s = "jiijpvbge2e3c3botuzzz7la3utpl67v"
         options1 = {"save-to": None,
-                   "verbose": None}
+                   "verbose": None,
+                   "timestamps": "short-local"}
         out = StringIO()
         lp = tail.LogPrinter(options1, target_tubid_s[:8], out)
         lp.got_versions({})
@@ -1552,7 +1606,8 @@ class Tail(unittest.TestCase):
     def test_logprinter_verbose(self):
         target_tubid_s = "jiijpvbge2e3c3botuzzz7la3utpl67v"
         options1 = {"save-to": None,
-                   "verbose": True}
+                   "verbose": True,
+                   "timestamps": "short-local"}
         out = StringIO()
         lp = tail.LogPrinter(options1, target_tubid_s[:8], out)
         lp.got_versions({})
@@ -1571,7 +1626,8 @@ class Tail(unittest.TestCase):
         target_tubid_s = "jiijpvbge2e3c3botuzzz7la3utpl67v"
         saveto_filename = "test_logprinter_saveto.flog"
         options = {"save-to": saveto_filename,
-                   "verbose": False}
+                   "verbose": False,
+                   "timestamps": "short-local"}
         out = StringIO()
         lp = tail.LogPrinter(options, target_tubid_s[:8], out)
         lp.got_versions({})
@@ -1635,7 +1691,7 @@ def run_wrapper(argv):
     config.parseOptions(argv)
     command = config.subCommand
     if command == "flogtool":
-        return cli.run_flogtool(argv[1:])
+        return cli.run_flogtool(argv[1:], run_by_human=False)
 
 class CLI(unittest.TestCase):
     def test_create_gatherer(self):
@@ -1748,6 +1804,7 @@ class Dumper(unittest.TestCase, LogfileWriterMixin, LogfileReaderMixin):
             # initialize the LogDumper() timestamp mode
             d.options = dumper.DumpOptions()
             d.options.parseOptions([fn])
+            tmode = d.options["timestamps"]
 
             argv = ["flogtool", "dump", fn]
             (out,err) = cli.run_flogtool(argv[1:], run_by_human=False)
@@ -1759,7 +1816,8 @@ class Dumper(unittest.TestCase, LogfileWriterMixin, LogfileReaderMixin):
             self.failUnlessEqual(lines[3].strip(), "PID: %s" % mypid, lines[3])
             lines = lines[5:]
             line0 = "local#%d %s: one" % (events[1]["d"]["num"],
-                                          d.format_time(events[1]["d"]["time"]))
+                                          format_time(events[1]["d"]["time"],
+                                                      tmode))
             self.failUnlessEqual(lines[0].strip(), line0)
             self.failUnless("FAILURE:" in lines[3])
             self.failUnless("test_logging.SampleError: whoops1" in lines[-3])
@@ -1769,7 +1827,7 @@ class Dumper(unittest.TestCase, LogfileWriterMixin, LogfileReaderMixin):
             (out,err) = cli.run_flogtool(argv[1:], run_by_human=False)
             self.failUnlessEqual(err, "")
             lines = list(StringIO(out).readlines())
-            line0 = "%s %d" % (d.format_time(events[1]["d"]["time"]),
+            line0 = "%s %d" % (format_time(events[1]["d"]["time"], tmode),
                                events[1]["d"]["num"])
             self.failUnlessEqual(lines[0].strip(), line0)
             self.failUnless(lines[1].strip().endswith(" 1"))
@@ -1788,8 +1846,8 @@ class Dumper(unittest.TestCase, LogfileWriterMixin, LogfileReaderMixin):
             lines = lines[5:]
             line0 = "local#%d rx(%s) emit(%s): one" % \
                     (events[1]["d"]["num"],
-                     d.format_time(events[1]["rx_time"]),
-                     d.format_time(events[1]["d"]["time"]))
+                     format_time(events[1]["rx_time"], tmode),
+                     format_time(events[1]["d"]["time"], tmode))
             self.failUnlessEqual(lines[0].strip(), line0)
             self.failUnless(lines[-1].strip().endswith(" four"))
 
@@ -1832,6 +1890,23 @@ class Dumper(unittest.TestCase, LogfileWriterMixin, LogfileReaderMixin):
         d.addCallback(_check)
         return d
 
+    def test_oops_furl(self):
+        self.basedir = "logging/Dumper/oops_furl"
+        if not os.path.exists(self.basedir):
+            os.makedirs(self.basedir)
+        fn = os.path.join(self.basedir, "logport.furl")
+        f = open(fn, "w")
+        f.write("pb://TUBID@HINTS/SWISSNUM\n")
+        f.close()
+
+        d = dumper.LogDumper()
+        # initialize the LogDumper() timestamp mode
+        d.options = dumper.DumpOptions()
+        d.options.parseOptions([fn])
+        argv = ["flogtool", "dump", fn]
+        (out,err) = cli.run_flogtool(argv[1:], run_by_human=False)
+        self.failUnlessEqual(err, "Error: logging/Dumper/oops_furl/logport.furl appears to be a FURL file.\nPerhaps you meant to run 'flogtool tail' instead of 'flogtool dump'?\n")
+
 class Filter(unittest.TestCase, LogfileWriterMixin, LogfileReaderMixin):
 
     def compare_events(self, a, b):
@@ -2033,7 +2108,7 @@ class Web(unittest.TestCase):
                       client.getPage(self.baseurl + "all-events?sort=nested"))
         d.addCallback(_check_all_events)
         d.addCallback(lambda res:
-                      client.getPage(self.baseurl + "all-events?timestamps=local"))
+                      client.getPage(self.baseurl + "all-events?timestamps=short-local"))
         d.addCallback(_check_all_events)
         d.addCallback(lambda res:
                       client.getPage(self.baseurl + "all-events?timestamps=utc"))
