@@ -2,7 +2,7 @@
 from twisted.trial import unittest
 from twisted.internet import defer
 from twisted.python import failure
-from foolscap import util, eventual
+from foolscap import util, eventual, base32
 
 
 class AsyncAND(unittest.TestCase):
@@ -90,3 +90,22 @@ class AsyncAND(unittest.TestCase):
         self.shouldFail()
 
 
+class Base32(unittest.TestCase):
+    def test_is_base32(self):
+        self.failUnless(base32.is_base32("abc456"))
+        self.failUnless(base32.is_base32("456"))
+        self.failUnless(base32.is_base32(""))
+        self.failIf(base32.is_base32("123")) # 1 is not in rfc4648 base32
+        self.failIf(base32.is_base32(".123"))
+        self.failIf(base32.is_base32("_"))
+        self.failIf(base32.is_base32("a b c"))
+
+class Time(unittest.TestCase):
+    def test_format(self):
+        when = 1339286175.7071271
+        self.failUnlessEqual(util.format_time(when, "utc"),
+                             "2012-06-09_23:56:15.707127Z")
+        self.failUnlessEqual(util.format_time(when, "epoch"),
+                             "1339286175.707")
+        self.failUnless(":" in util.format_time(when, "short-local"))
+        self.failUnless(":" in util.format_time(when, "long-local"))
